//
//  OCHamcrest - IsDictionaryContainingValueTest.m
//  Copyright 2009 www.hamcrest.org. See LICENSE.txt
//
//  Created by: Jon Reid
//

    // Inherited
#import "AbstractMatcherTest.h"

    // OCHamcrest
#define HC_SHORTHAND
#import "HCIsDictionaryContainingValue.h"
#import "HCIsEqual.h"


@interface IsDictionaryContainingValueTest : AbstractMatcherTest
@end

@implementation IsDictionaryContainingValueTest

- (id<HCMatcher>) createMatcher
{
    return hasValue(@"irrelevant");
}


- (void) testHasReadableDescription
{
    assertDescription(@"dictionary with value \"a\"", hasValue(@"a"));
}


- (void) testDoesNotMatchEmptyDictionary
{
    assertDoesNotMatch(@"Empty dictionary", hasValue(@"Foo"), [NSDictionary dictionary]);
}


- (void) testMatchesSingletonDictionaryContainingValue
{
    NSDictionary* dict = [NSDictionary dictionaryWithObject:@"1" forKey:@"a"];
    
    assertMatches(@"Matches single key", hasValue(equalTo(@"1")), dict);
}


- (void) testMatchesDictionaryContainingValue
{
    NSDictionary* dict = [NSDictionary dictionaryWithObjectsAndKeys:
                                            @"1", @"a",
                                            @"2", @"b",
                                            @"3", @"c",
                                            nil];
    
    assertMatches(@"Matches 1", hasValue(equalTo(@"1")), dict);
    assertMatches(@"Matches 3", hasValue(equalTo(@"3")), dict);
}


- (void) testProvidesConvenientShortcutForMatchingWithIsEqualTo
{
    NSDictionary* dict = [NSDictionary dictionaryWithObjectsAndKeys:
                                            @"1", @"a",
                                            @"2", @"b",
                                            @"3", @"c",
                                            nil];

    assertMatches(@"Matches 3", hasValue(@"3"), dict);
}


- (void) testDoesNotMatchDictionaryMissingValue
{
    NSDictionary* dict = [NSDictionary dictionaryWithObjectsAndKeys:
                                            @"1", @"a",
                                            @"2", @"b",
                                            @"3", @"c",
                                            nil];
    
    assertDoesNotMatch(@"Dictionary without matching value", hasValue(@"4"), dict);
}


@end
